#include "BBBThread.h"

BBBThread* Runable::GetThread()
{
	return m_pThread;
}

// Inplementation af staticmetode for BBBThread

// Bemrk static-metode - "binder" til objektmetode
// Denne fr pointer til objektet med som parameter og
// kan s kalde videre til en objektmetoden Run
DWORD WINAPI BBBThread::StaticRun(LPVOID param)
{
	// typecast parameter til traadobjekt pointer
	BBBThread *pThreadObject = (BBBThread*)param;
	// "vidrestil" til objektmetode
	pThreadObject->m_Running = TRUE;
	DWORD ret = pThreadObject->m_pRunObj->Run();
	pThreadObject->m_Running = FALSE;
	return ret;
}

// Inplementation af objektmetoder for BBBThread

DWORD BBBThread::Run()
{
	cout << "Error - run metode er ikke implementeret" << endl;
	exit(1);
}

// constructorer
BBBThread::BBBThread(Runable *pRunableObj, const string threadName)
:	m_pRunObj(pRunableObj),m_HandleThread(NULL), m_ThreadId(0),
	m_Stop(FALSE), m_Suspended(FALSE), m_ThreadName(threadName)
{
	if (!m_pRunObj)
	{
		cout << "Error - Object med Run-metode mangler" << endl;
		exit(1);
	}
	m_pRunObj->m_pThread = this;
}

BBBThread::BBBThread(const string threadName)
:	m_HandleThread(NULL), m_ThreadId(0),
	m_Stop(FALSE), m_Suspended(FALSE), m_ThreadName(threadName)
{
	m_pRunObj = this;
	m_pRunObj->m_pThread = this;
}

// Destructor
BBBThread::~BBBThread(void)
{
	if (m_Running)
	{
		cout << "Error - Destructer called on running Thread object" << endl;
		exit(1);
	}
}

// Andre Thread-metoder
void BBBThread::Start()
{
	m_HandleThread = (HANDLE)_beginthreadex
						(
							NULL,		// Security
							2000,		// stacksize
							(PBEGINTHREADEX_THREADFUNC) StaticRun,	//"Binde-metode"
							(LPVOID) this,					// parameter=objekt
							0,			// initflag
							(PBEGINTHREADEX_THREADID) &m_ThreadId	// id returneres
						);
	if (!m_HandleThread)
	{
		cout << "Error - Thread not started" << endl;
		exit(1);
	}
}

void BBBThread::Join(DWORD waitMilliseconds)
{
	WaitForSingleObject(m_HandleThread,waitMilliseconds);
	CloseHandle(m_HandleThread);
}

void BBBThread::SetSignalStop()
{
	m_Stop = TRUE;
}

BOOL BBBThread::GetStopSignal()
{
	return m_Stop;
}

BOOL BBBThread::IsRunning()
{
	return m_Running;
}

const string BBBThread::GetName()
{
	return m_ThreadName;
}

void BBBThread::Suspend()
{
	DWORD suspendCount = SuspendThread(m_pThread);
	if (suspendCount==0xFFFFFFFF)
		cout << "Error - ved suspend af traad "<<GetName().data()<<endl;
	else
		m_Suspended=TRUE;	//NB: Problem hvis paralel Resume nr frst
}

void BBBThread::Resume()
{
	DWORD suspendCount;
	do {
		suspendCount =ResumeThread(m_pThread);
	}while(suspendCount>0);	// fortst til suspend-counter=0
	if (suspendCount==0xFFFFFFFF)
		cout << "Error - ved resupe af traad "<<GetName().data()<<endl;
	else
		m_Suspended=FALSE;	//NB: Problem hvis paralel Suspend nr frst
}

BOOL BBBThread::IsSuspended()
{
	return (m_Suspended);
}
